//
//  tank
//
//  Import and Playlist Daemon for autoradio project
//
//
//  Copyright (C) 2017-2019 Christian Pointner <equinox@helsinki.at>
//
//  This file is part of tank.
//
//  tank is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  any later version.
//
//  tank is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with tank. If not, see <http://www.gnu.org/licenses/>.
//

package auth

import (
	"net/http"
	"strings"

	"github.com/gorilla/mux"
)

var auth = &Auth{}

type Auth struct {
	sessions *SessionManager
	oidc     *OIDCBackend
}

func Init(c *Config) (err error) {
	if auth.sessions, err = NewSessionManager(c.Sessions); err != nil {
		return
	}

	if c.OIDC != nil {
		if auth.oidc, err = NewOIDCBackend(c.OIDC); err != nil {
			return
		}
	}

	return
}

func Login(w http.ResponseWriter, r *http.Request) {
	method := r.URL.Query().Get("method")
	switch strings.ToLower(method) {
	case "oidc":
		if auth.oidc == nil {
			http.Error(w, "OIDC authentication is not configured", http.StatusBadRequest)
			return
		}
		auth.oidc.HandleLogin(w, r)
	case "":
		http.Error(w, "default/fallback authentication method has not been implemented yet!", http.StatusNotImplemented)
	default:
		http.Error(w, "invalid authentication method: "+method, http.StatusBadRequest)
	}
}

func InstallHTTPHandler(r *mux.Router) {
	r.HandleFunc("/login", Login)
	if auth.oidc != nil {
		r.Handle("/oidc/callback", auth.oidc.CallbackHandler())
	}
}
